@@ -66,8 +66,7 @@ def _fix_input_json(
 
 class CelloResult:
     def __init__(self, results_dir: str):
-        """
-        High Level Class that encapsulates some of the oddities of
+        """High Level Class that encapsulates some of the oddities of
         recapitulating the necessary information from the Cello output
         documentation.
 
@@ -100,10 +99,9 @@ class CelloResult:
         self,
         filename: str,
     ) -> Dict[str, Any]:
-        """
-        Utility function to pull in a CSV and convert it into a dictionary where
-        the keys are the node labels and the values correspond to whichever CSV
-        is being requested.
+        """Utility function to pull in a CSV and convert it into a dictionary
+        where the keys are the node labels and the values correspond to
+        whichever CSV is being requested.
 
         Numpy CSV parsing is due to future optimization concerns.
 
@@ -114,7 +112,8 @@ class CelloResult:
             A dictionary composed of node label keys and whatever the requested
             data is.
         """
-        in_file = glob.glob(f"{self.output_dir}/*_{filename}.csv")
+        fp = os.path.join(f"{self.output_dir}", f"*_{filename}.csv")
+        in_file = glob.glob(fp)
         if not in_file:
             raise RuntimeError(
                 f"Unable to find {filename}.csv. Please check output directory."
@@ -136,10 +135,9 @@ class CelloResult:
         return mapping_keys
 
     def _map_symbolic_representation_to_true_name(self) -> Dict[str, str]:
-        """
-        All internal references in all of the Cello results reference everything
-        by it's associated verilog variable name, but that's not parsable by
-        human beings.
+        """All internal references in all of the Cello results reference
+        everything by it's associated verilog variable name, but that's not
+        parsable by human beings.
 
         This maps those variables to their human names.
 
@@ -148,7 +146,8 @@ class CelloResult:
             biological nomenclature, e.g. yfp_cassette, Phlf, etc.
 
         """
-        in_file = glob.glob(f"{self.output_dir}/*_outputNetlist.json")
+        fp = os.path.join(f'{self.output_dir}', '*_outputNetlist.json')
+        in_file = glob.glob(fp)
         if not in_file:
             raise RuntimeError(
                 "Unable to find Output Netlist. Please check output directory."
@@ -167,8 +166,7 @@ class CelloResult:
         return out_mapping
 
     def score_repressors(self) -> Dict[str, float]:
-        """
-        Scores all of the repressors in the genetic circuit according to the
+        """Scores all of the repressors in the genetic circuit according to the
         highest off and lowest on.
 
         Returns:
@@ -194,11 +192,7 @@ class CelloResult:
                 [i for idx, i in enumerate(activity_values) if truth_table[idx]]
             )
             high_off = max(
-                [
-                    i
-                    for idx, i in enumerate(activity_values)
-                    if not truth_table[idx]
-                ]
+                [i for idx, i in enumerate(activity_values) if not truth_table[idx]]
             )
             score = math.log(low_on / high_off)
             score_dict[r_key] = score
@@ -225,7 +219,8 @@ class CelloResult:
         """
         # This is probably a code smell, given that this information is vitally
         # important but only captures in a STDOUT Log. TODO, I suppose.
-        in_file = glob.glob(f"{self.output_dir}/*.log")
+        fp = os.path.join(f'{self.output_dir}', '*.log')
+        in_file = glob.glob(fp)
         if len(in_file) - 1:
             raise RuntimeError(
                 "Found multiple results matching pattern. Please investigate."
@@ -253,6 +248,7 @@ class CelloQuery:
         input_sensors: str = None,
         output_device: str = None,
         logging: bool = False,
+        archival: bool = True,
     ):
         """
         Class encapsulating all of the weirdness of interacting with Cello.
@@ -275,7 +271,14 @@ class CelloQuery:
             logging: Optional. If true, when results are submitted the terminal
                 output from the docker container will be displayed in the
                 terminal.
+            archival: Optional. If true, when results are submitted to the
+                docker container the prior results are moved to a new folder to
+                prevent clobbering. If false, we remove the previous results.
         """
+        self.check_dependencies()
+        if sys.platform == "win32":
+            input_directory = input_directory.lower()
+            output_directory = output_directory.lower()
         self.input_directory = input_directory
         self.output_directory = output_directory
         self.verilog_file = verilog_file
@@ -284,12 +287,73 @@ class CelloQuery:
         self.input_sensors = input_sensors
         self.original_input_sensors = input_sensors
         self.output_device = output_device
+
         self.logging = logging
+        self.archival = archival
+
+    def check_paths(self):
+        dir_attributes = [
+            self.input_directory,
+            self.output_directory
+        ]
+        file_attributes = [
+            self.verilog_file,
+            self.compiler_options,
+            self.input_ucf,
+            self.input_sensors,
+            self.output_device,
+        ]
+        for directory in dir_attributes:
+            if not os.path.isdir(directory):
+                raise RuntimeError(
+                    f'Unable to find {directory}. Please Investigate.'
+                )
+        for file in file_attributes:
+            fp = os.path.join(self.input_directory, file)
+            if not os.path.isfile(fp):
+                raise RuntimeError(
+                    f'Unable to find {file}. Please Investigate.'
+                )
+
+    @staticmethod
+    def check_dependencies():
+        output = None
+        negative_check = None
+        if sys.platform == "win32":
+            docker_check = f"docker"
+            output = subprocess.run(
+                [docker_check],
+                capture_output=True,
+                text=True,
+            ).stdout
+            negative_check = (
+                "'docker' is not recognized as an internal or external command, "
+                "operable program or batch file."
+            )
+        if sys.platform == "linux" or sys.platform == "darwin":
+            docker_check = f"docker"
+            output = subprocess.run(
+                [docker_check],
+                capture_output=True,
+                text=True,
+            ).stdout
+            negative_check = (
+                "The program 'docker' is currently not installed. You can "
+                "install it by typing: "
+                "apt-get install docker"
+            )
+        if output is None or negative_check is None:
+            raise RuntimeError("Unable to detect Operating System, exiting.")
+        if output == negative_check:
+            raise RuntimeError(
+                "Docker not Found. Please install Docker. "
+                "(https://docs.docker.com/get-docker/) "
+            )
 
     def get_results(self) -> int:
         """
         Primary Entrypoint for the library. Wraps calling the Cello Docker
-        Container and takes care of all of the busywork neccesary to make
+        Container and takes care of all of the busywork necessary to make
         everything come together.
 
         Returns:
@@ -305,32 +369,60 @@ class CelloQuery:
         # Probably some check to see if the directories are being generated
         # properly.
         if self.check_for_prior_results():
-            print("Prior results detected, moving to prevent clobber.")
-            dest = self.archive_prior_results()
-            print(f"Prior results move to {dest}")
-        docker_cmd = [
-            f"docker run --rm "
-            f"-v {self.input_directory}:/root/input "
-            f"-v {self.output_directory}:/root/output"
-            f" -t cidarlab/cello-dnacompiler:latest java -classpath /root/app.jar "
-            f"org.cellocad.v2.DNACompiler.runtime.Main "
-            f"-inputNetlist /root/input/{self.verilog_file} "
-            f"-options /root/input/{self.compiler_options} "
-            f"-userConstraintsFile /root/input/{self.input_ucf} "
-            f"-inputSensorFile /root/input/{self.input_sensors} "
-            f"-outputDeviceFile /root/input/{self.output_device} "
-            f"-pythonEnv python "
-            f"-outputDir /root/output"
-        ]
+            if self.archival:
+                print("Prior results detected, moving to prevent clobber.")
+                dest = self.archive_prior_results()
+                print(f"Prior results move to {dest}")
+            if not self.archival:
+                print("Prior results detected, removing prior results.")
+                self.remove_prior_results()
+                print(f"Prior results deleted.")
+        docker_cmd = None
+        if sys.platform == "linux" or sys.platform == "darwin":
+            docker_cmd = [
+                f"docker run --rm "
+                f"-v {self.input_directory}:/root/input "
+                f"-v {self.output_directory}:/root/output"
+                f" -t cidarlab/cello-dnacompiler:latest java -classpath /root/app.jar "
+                f"org.cellocad.v2.DNACompiler.runtime.Main "
+                f"-inputNetlist /root/input/{self.verilog_file} "
+                f"-options /root/input/{self.compiler_options} "
+                f"-userConstraintsFile /root/input/{self.input_ucf} "
+                f"-inputSensorFile /root/input/{self.input_sensors} "
+                f"-outputDeviceFile /root/input/{self.output_device} "
+                f"-pythonEnv python "
+                f"-outputDir /root/output"
+            ]
+        if sys.platform == "win32":
+            docker_cmd = (
+                f"docker run --rm "
+                + f"-v {self.input_directory}:/root/input "
+                + f"-v {self.output_directory}:/root/output"
+                + f" -t cidarlab/cello-dnacompiler:latest java -classpath /root/app.jar "
+                + f"org.cellocad.v2.DNACompiler.runtime.Main "
+                + f"-inputNetlist /root/input/{self.verilog_file} "
+                + f"-options /root/input/{self.compiler_options} "
+                + f"-userConstraintsFile /root/input/{self.input_ucf} "
+                + f"-inputSensorFile /root/input/{self.input_sensors} "
+                + f"-outputDeviceFile /root/input/{self.output_device} "
+                + f"-pythonEnv python "
+                + f"-outputDir /root/output"
+            )
+        if docker_cmd is None:
+            raise RuntimeError(
+                "Unable to detect proper operating system to create docker "
+                "subcommand stings. Please investigate."
+            )
         try:
             process = subprocess.Popen(
-                docker_cmd, shell=True, stdout=subprocess.PIPE
-            )
+                docker_cmd,
+                shell=True,
+                stdout=subprocess.PIPE)
             if self.logging:
                 try:
                     for line in iter(lambda: process.stdout.read(1), b""):
                         sys.stdout.buffer.write(line)
-                except AttributError:
+                except AttributeError:
                     for line in iter(lambda: process.stdout.read(1), b""):
                         sys.stdout.write(line)
             else:
@@ -355,7 +447,8 @@ class CelloQuery:
         Returns:
             A list of human readable input signals.
         """
-        data = _fix_input_json(f"{self.input_directory}/{self.input_sensors}")
+        sig_path = os.path.join(self.input_directory, self.input_sensors)
+        data = _fix_input_json(sig_path)
         # Each input sensor is represented by a disjoint triplet of
         # an input sensor datastructure, a model of that input sensor, and one
         # representing the structure.
@@ -411,12 +504,8 @@ class CelloQuery:
         # Once we're good we need to trim the input sensors to just what is
         # being used.
         data = _fix_input_json(f"{self.input_directory}/{self.input_sensors}")
-        data = _prune_other_sensors(
-            "_sensor", "input_sensors", data, input_signals
-        )
-        data = _prune_other_sensors(
-            "_sensor_model", "models", data, input_signals
-        )
+        data = _prune_other_sensors("_sensor", "input_sensors", data, input_signals)
+        data = _prune_other_sensors("_sensor_model", "models", data, input_signals)
         data = _prune_other_sensors(
             "_sensor_structure", "structures", data, input_signals
         )
@@ -427,6 +516,7 @@ class CelloQuery:
         return output_filename
 
     def reset_input_signals(self):
+        """Sets the input signals back to what they were originally."""
         self.input_sensors = self.original_input_sensors
 
     def check_for_prior_results(self) -> bool:
@@ -436,7 +526,8 @@ class CelloQuery:
         Returns:
             If there are prior results in the output directory.
         """
-        dir_contents = glob.glob(f"{self.output_directory}/*")
+        dir_path = os.path.join(f"{self.output_directory}", '*')
+        dir_contents = glob.glob(dir_path)
         dir_contents = list(
             filter(lambda x: "prior_cello_result" not in x, dir_contents)
         )
@@ -452,9 +543,9 @@ class CelloQuery:
             The location that the prior results were move to.
 
         """
-        # TODO: Not sure how to do compression that's safe across all OSes.
-        archive_dir_name = f"prior_cello_result_{datetime.datetime.now()}"
-        archive_dir = f"{self.output_directory}/{archive_dir_name}"
+        time_string = datetime.datetime.now().strftime("%Y-%m-%d_%H.%M.%S")
+        archive_dir_name = f"prior_cello_result_{time_string}"
+        archive_dir = os.path.join(self.output_directory, archive_dir_name)
         os.mkdir(archive_dir)
         active_files = glob.glob(f"{self.output_directory}/*")
         active_files = list(
@@ -464,5 +555,20 @@ class CelloQuery:
             try:
                 shutil.move(file, archive_dir)
             except PermissionError:
-                print(f'Failed to move {file}')
+                print(f"Failed to move {file}")
         return archive_dir
+
+    def remove_prior_results(self):
+        """
+        Removes prior results to prevent aggregation of files.
+        """
+        fp = os.path.join(f'{self.output_directory}', '*')
+        active_files = glob.glob(fp)
+        active_files = list(
+            filter(lambda x: "prior_cello_result" not in x, active_files)
+        )
+        for file in active_files:
+            try:
+                os.remove(file)
+            except PermissionError:
+                print(f"Failed to delete {file}. Please check permissions")
